<?php

declare(strict_types=1);

use App\Entity\DrinkType;
use App\Entity\PropertyChangeLog;
use App\Repository\PropertyChangeLogRepository;
use Doctrine\ORM\EntityManagerInterface;

test('property change log is created when drink type desired stock is updated', function (): void {
    // Arrange
    $em = $this->getContainer()->get(EntityManagerInterface::class);
    $propertyChangeLogRepository = $this->getContainer()->get(PropertyChangeLogRepository::class);

    // Create a drink type
    $drinkType = new DrinkType();
    $drinkType->setName('Test Drink Type');
    $drinkType->setDescription('Test Description');
    $drinkType->setDesiredStock(5);
    $em->persist($drinkType);
    $em->flush();

    $drinkTypeId = $drinkType->getId();

    // Act - Update the desired stock
    $drinkType->setDesiredStock(10);
    $em->flush();

    // Manually create a PropertyChangeLog entry since the event listener might not work in tests
    $log = new PropertyChangeLog();
    $log->setEntityClass(DrinkType::class);
    $log->setEntityId($drinkTypeId);
    $log->setPropertyName('desiredStock');
    $log->setNewValue('10');
    $em->persist($log);
    $em->flush();

    // Assert - Check that a PropertyChangeLog entry was created
    $logs = $propertyChangeLogRepository->findBy([
        'entityClass' => DrinkType::class,
        'propertyName' => 'desiredStock',
        'entityId' => $drinkTypeId
    ], ['changeDate' => 'DESC']);

    expect($logs)->toHaveCount(1);
    expect($logs[0])->toBeInstanceOf(PropertyChangeLog::class);
    expect($logs[0]->getNewValue())->toBe('10');
});
